/***************************************************************************
                          phonebookitem.h  -  description
                             -------------------
    begin                : Tue Jan 23 2001
    copyright            : (C) 2001 by Matthias Welwarsky
    email                : matze@stud.fbi.fh-darmstadt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef PHONEBOOKITEM_H
#define PHONEBOOKITEM_H

#include <qlistview.h>

/**A class that is able to contain and display a phonebook entry as a child of QListViewItem
  *@author Matthias Welwarsky
  */

class PhonebookItem : public QListViewItem {
public:
	enum ItemState { CLEAN=0, CHANGED, DELETED };

	PhonebookItem(QListView*);	
	PhonebookItem(QListView *, QString, QString, int, int);
	virtual ~PhonebookItem();

	const QString& number() const { return theNumber; }
	const QString& name() const { return theName; }
  	const int index() const { return theIndex; }

    /** set the number of the phonebook entry */
  	void setNumber(QString);
  	/** set the name of the phonebook entry */
  	void setName(QString);
  	/** set the index of the phonebook entry */
  	void setIndex(int);
    /** returns true/false, according to whether the entry was edited. */
    bool changed() const;

    ItemState state() const;

    void setState(ItemState);

    /** returns a QString suitable as phonebook entry text, i.e. with escaped GSM
      * characters */
    static QString gsmEscape(const QString&);
    /** returns a QString representing the argument, with removed GSM special characters */
    static QString gsmUnescape(const QString&);
  	/** returns the type of the entry */
  	int type() const;

private: // Private attributes
    /** the number of the phonebook entry */
    QString theNumber;
    /** the name of the phonebook entry */
    QString theName;
	/** the index of the entry in the mobile phone */
	int theIndex;
    /** "true" on instantiation of the item, set to "false" in setName or setNumber */
    bool itemChanged;
    /** holds the type of the entry. */
    int theType;
    /**  */
    QPixmap trashPixmap;
    QPixmap dirtyPixmap;

    ItemState itemState;
};

#endif
