/***************************************************************************
                          phoneterminal.cpp  -  description
                             -------------------
    begin                : Fri Jan 19 2001
    copyright            : (C) 2001 by Matthias Welwarsky
    email                : matze@stud.fbi.fh-darmstadt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>

#include <qregexp.h>
#include <qlineedit.h>
#include <qtextview.h>

#include "phoneterminal.h"
#include "phonectrlcenter.h"
#include "messagedispatcher.h"

PhoneTerminal::PhoneTerminal(QWidget *parent, const char *name ) : PhoneTerminalDlg(parent,name)
{
	PhoneCtrlCenter* pcCenter = dynamic_cast<PhoneCtrlCenter*>(parentWidget());
	if (!pcCenter) {
		kdDebug() << "cannot resolve control center\n";
		return;
	}

	messageDispatcher = pcCenter->getDispatcher();

	// designer should have done this, but it didn't! BUG!
	connect((QObject*)transmitEdit, SIGNAL(returnPressed()), (QObject*)this, SLOT(commandReady()));
	connect((QObject*)escButton, SIGNAL(clicked()), (QObject*)this, SLOT(sendEscape()));
}

PhoneTerminal::~PhoneTerminal(){
}

/** reimplemented from PhoneControl */
void PhoneTerminal::startControl(){
	messageDispatcher->acquireDispatcher(this, SLOT(deliverMessage(QString)));
}

/** reimplemented from PhoneControl */
void PhoneTerminal::stopControl(){
	messageDispatcher->releaseDispatcher(this);
}

/** reimplemented from PhoneControl */
void PhoneTerminal::deliverMessage(QString message){
	receiveView->append(message);
}

/** called by the transmitEdit when the enter key is pressed */
void PhoneTerminal::commandReady(){
	messageDispatcher->sendCommand(transmitEdit->text(), this);
}

/** sends an escape character to the phone */
void PhoneTerminal::sendEscape(){
	messageDispatcher->sendPDU(transmitEdit->text(), this);
}
